@@ -8,25 +8,30 @@ from textwrap import dedent, wrap
 from httpie import __doc__, __version__
 from httpie.cli.argparser import HTTPieArgumentParser
 from httpie.cli.argtypes import (
-    KeyValueArgType, SessionNameValidator, readable_file_arg,
+    KeyValueArgType, SessionNameValidator,
+    readable_file_arg,
 )
 from httpie.cli.constants import (
-    OUTPUT_OPTIONS, OUTPUT_OPTIONS_DEFAULT, OUT_REQ_BODY, OUT_REQ_HEAD,
+    DEFAULT_FORMAT_OPTIONS, OUTPUT_OPTIONS,
+    OUTPUT_OPTIONS_DEFAULT, OUT_REQ_BODY, OUT_REQ_HEAD,
     OUT_RESP_BODY, OUT_RESP_HEAD, PRETTY_MAP, PRETTY_STDOUT_TTY_ONLY,
-    SEPARATOR_GROUP_ALL_ITEMS, SEPARATOR_PROXY, SSL_VERSION_ARG_MAPPING,
+    RequestType, SEPARATOR_GROUP_ALL_ITEMS, SEPARATOR_PROXY,
+    SORTED_FORMAT_OPTIONS_STRING,
+    UNSORTED_FORMAT_OPTIONS_STRING,
 )
 from httpie.output.formatters.colors import (
     AUTO_STYLE, AVAILABLE_STYLES, DEFAULT_STYLE,
 )
-from httpie.plugins import plugin_manager
 from httpie.plugins.builtin import BuiltinAuthPlugin
+from httpie.plugins.registry import plugin_manager
 from httpie.sessions import DEFAULT_SESSIONS_DIR
+from httpie.ssl import AVAILABLE_SSL_VERSION_ARG_MAPPING, DEFAULT_SSL_CIPHERS
 
 
 parser = HTTPieArgumentParser(
     prog='http',
     description='%s <https://httpie.org>' % __doc__.strip(),
-    epilog=dedent("""
+    epilog=dedent('''
     For every --OPTION there is also a --no-OPTION that reverts OPTION
     to its default value.
 
@@ -34,7 +39,7 @@ parser = HTTPieArgumentParser(
 
         https://github.com/jakubroztocil/httpie/issues
 
-    """),
+    '''),
 )
 
 #######################################################################
@@ -43,18 +48,18 @@ parser = HTTPieArgumentParser(
 
 positional = parser.add_argument_group(
     title='Positional Arguments',
-    description=dedent("""
+    description=dedent('''
     These arguments come after any flags and in the order they are listed here.
     Only URL is required.
 
-    """)
+    ''')
 )
 positional.add_argument(
     dest='method',
     metavar='METHOD',
     nargs=OPTIONAL,
     default=None,
-    help="""
+    help='''
     The HTTP method to be used for the request (GET, POST, PUT, DELETE, ...).
 
     This argument can be omitted in which case HTTPie will use POST if there
@@ -63,12 +68,12 @@ positional.add_argument(
         $ http example.org               # => GET
         $ http example.org hello=world   # => POST
 
-    """
+    '''
 )
 positional.add_argument(
     dest='url',
     metavar='URL',
-    help="""
+    help='''
     The scheme defaults to 'http://' if the URL does not include one.
     (You can override this with: --default-scheme=https)
 
@@ -77,7 +82,7 @@ positional.add_argument(
         $ http :3000                    # => http://localhost:3000
         $ http :/foo                    # => http://localhost/foo
 
-    """
+    '''
 )
 positional.add_argument(
     dest='request_items',
@@ -85,7 +90,7 @@ positional.add_argument(
     nargs=ZERO_OR_MORE,
     default=None,
     type=KeyValueArgType(*SEPARATOR_GROUP_ALL_ITEMS),
-    help=r"""
+    help=r'''
     Optional key-value pairs to be included in the request. The separator used
     determines the type:
 
@@ -106,9 +111,10 @@ positional.add_argument(
 
         awesome:=true  amount:=42  colors:='["red", "green", "blue"]'
 
-    '@' Form file fields (only with --form, -f):
+    '@' Form file fields (only with --form or --multipart):
 
-        cs@~/Documents/CV.pdf
+        cv@~/Documents/CV.pdf
+        cv@'~/Documents/CV.pdf;type=application/pdf'
 
     '=@' A data field like '=', but takes a file path and embeds its content:
 
@@ -122,7 +128,7 @@ positional.add_argument(
 
         field-name-with\:colon=value
 
-    """
+    '''
 )
 
 #######################################################################
@@ -136,26 +142,50 @@ content_type = parser.add_argument_group(
 
 content_type.add_argument(
     '--json', '-j',
-    action='store_true',
-    help="""
+    action='store_const',
+    const=RequestType.JSON,
+    dest='request_type',
+    help='''
     (default) Data items from the command line are serialized as a JSON object.
     The Content-Type and Accept headers are set to application/json
     (if not specified).
 
-    """
+    '''
 )
 content_type.add_argument(
     '--form', '-f',
-    action='store_true',
-    help="""
+    action='store_const',
+    const=RequestType.FORM,
+    dest='request_type',
+    help='''
     Data items from the command line are serialized as form fields.
 
     The Content-Type is set to application/x-www-form-urlencoded (if not
     specified). The presence of any file fields results in a
     multipart/form-data request.
 
-    """
+    '''
 )
+content_type.add_argument(
+    '--multipart',
+    action='store_const',
+    const=RequestType.MULTIPART,
+    dest='request_type',
+    help='''
+    Similar to --form, but always sends a multipart/form-data
+    request (i.e., even without files).
+
+    '''
+)
+content_type.add_argument(
+    '--boundary',
+    help='''
+    Specify a custom boundary string for multipart/form-data requests.
+    Only has effect only together with --form.
+
+    '''
+)
+
 
 #######################################################################
 # Content processing.
@@ -170,14 +200,14 @@ content_processing.add_argument(
     '--compress', '-x',
     action='count',
     default=0,
-    help="""
+    help='''
     Content compressed (encoded) with Deflate algorithm.
     The Content-Encoding header is set to deflate.
 
     Compression is skipped if it appears that compression ratio is
     negative. Compression can be forced by repeating the argument.
 
-    """
+    '''
 )
 
 #######################################################################
@@ -191,12 +221,12 @@ output_processing.add_argument(
     dest='prettify',
     default=PRETTY_STDOUT_TTY_ONLY,
     choices=sorted(PRETTY_MAP.keys()),
-    help="""
+    help='''
     Controls output processing. The value can be "none" to not prettify
     the output (default for redirected output), "all" to apply both colors
     and formatting (default for terminal output), "colors", or "format".
 
-    """
+    '''
 )
 output_processing.add_argument(
     '--style', '-s',
@@ -204,10 +234,10 @@ output_processing.add_argument(
     metavar='STYLE',
     default=DEFAULT_STYLE,
     choices=AVAILABLE_STYLES,
-    help="""
-    Output coloring style (default is "{default}"). One of:
+    help='''
+    Output coloring style (default is "{default}"). It can be One of:
 
-{available_styles}
+    {available_styles}
 
     The "{auto_style}" style follows your terminal's ANSI color styles.
 
@@ -215,15 +245,75 @@ output_processing.add_argument(
     $TERM environment variable is set to "xterm-256color" or similar
     (e.g., via `export TERM=xterm-256color' in your ~/.bashrc).
 
-    """.format(
+    '''.format(
         default=DEFAULT_STYLE,
         available_styles='\n'.join(
             '{0}{1}'.format(8 * ' ', line.strip())
             for line in wrap(', '.join(sorted(AVAILABLE_STYLES)), 60)
-        ).rstrip(),
+        ).strip(),
         auto_style=AUTO_STYLE,
     )
 )
+_sorted_kwargs = {
+    'action': 'append_const',
+    'const': SORTED_FORMAT_OPTIONS_STRING,
+    'dest': 'format_options'
+}
+_unsorted_kwargs = {
+    'action': 'append_const',
+    'const': UNSORTED_FORMAT_OPTIONS_STRING,
+    'dest': 'format_options'
+}
+# The closest approx. of the documented resetting to default via --no-<option>.
+# We hide them from the doc because they act only as low-level aliases here.
+output_processing.add_argument('--no-unsorted', **_sorted_kwargs, help=SUPPRESS)
+output_processing.add_argument('--no-sorted', **_unsorted_kwargs, help=SUPPRESS)
+
+output_processing.add_argument(
+    '--unsorted',
+    **_unsorted_kwargs,
+    help=f'''
+    Disables all sorting while formatting output. It is a shortcut for:
+
+        --format-options={UNSORTED_FORMAT_OPTIONS_STRING}
+
+    '''
+)
+output_processing.add_argument(
+    '--sorted',
+    **_sorted_kwargs,
+    help=f'''
+    Re-enables all sorting options while formatting output. It is a shortcut for:
+
+        --format-options={SORTED_FORMAT_OPTIONS_STRING}
+
+    '''
+)
+
+
+output_processing.add_argument(
+    '--format-options',
+    action='append',
+    help='''
+    Controls output formatting. Only relevant when formatting is enabled
+    through (explicit or implied) --pretty=all or --pretty=format.
+    The following are the default options:
+
+        {option_list}
+
+    You may use this option multiple times, as well as specify multiple
+    comma-separated options at the same time. For example, this modifies the
+    settings to disable the sorting of JSON keys, and sets the indent size to 2:
+
+        --format-options json.sort_keys:false,json.indent:2
+
+    This is something you will typically put into your config file.
+
+    '''.format(
+        option_list='\n'.join(
+            (8 * ' ') + option for option in DEFAULT_FORMAT_OPTIONS).strip()
+    )
+)
 
 #######################################################################
 # Output options
@@ -234,7 +324,7 @@ output_options.add_argument(
     '--print', '-p',
     dest='output_options',
     metavar='WHAT',
-    help=f"""
+    help=f'''
     String specifying what the output should contain:
 
         '{OUT_REQ_HEAD}' request headers
@@ -247,70 +337,70 @@ output_options.add_argument(
     If the output is piped to another program or to a file, then only the
     response body is printed by default.
 
-    """
+    '''
 )
 output_options.add_argument(
     '--headers', '-h',
     dest='output_options',
     action='store_const',
     const=OUT_RESP_HEAD,
-    help=f"""
+    help=f'''
     Print only the response headers. Shortcut for --print={OUT_RESP_HEAD}.
 
-    """
+    '''
 )
 output_options.add_argument(
     '--body', '-b',
     dest='output_options',
     action='store_const',
     const=OUT_RESP_BODY,
-    help=f"""
+    help=f'''
     Print only the response body. Shortcut for --print={OUT_RESP_BODY}.
 
-    """
+    '''
 )
 
 output_options.add_argument(
     '--verbose', '-v',
     dest='verbose',
     action='store_true',
-    help="""
+    help='''
     Verbose output. Print the whole request as well as the response. Also print
     any intermediary requests/responses (such as redirects).
     It's a shortcut for: --all --print={0}
 
-    """.format(''.join(OUTPUT_OPTIONS))
+    '''.format(''.join(OUTPUT_OPTIONS))
 )
 output_options.add_argument(
     '--all',
     default=False,
     action='store_true',
-    help="""
+    help='''
     By default, only the final request/response is shown. Use this flag to show
     any intermediary requests/responses as well. Intermediary requests include
     followed redirects (with --follow), the first unauthorized request when
     Digest auth is used (--auth=digest), etc.
 
-    """
+    '''
 )
 output_options.add_argument(
     '--history-print', '-P',
     dest='output_options_history',
     metavar='WHAT',
-    help="""
+    help='''
     The same as --print, -p but applies only to intermediary requests/responses
     (such as redirects) when their inclusion is enabled with --all. If this
     options is not specified, then they are formatted the same way as the final
     response.
 
-    """
+    '''
 )
 output_options.add_argument(
     '--stream', '-S',
     action='store_true',
     default=False,
-    help="""
-    Always stream the output by line, i.e., behave like `tail -f'.
+    help='''
+    Always stream the response body by line, i.e., behave like `tail -f'.
 
     Without --stream and with --pretty (either set or implied),
     HTTPie fetches the whole response before it outputs the processed data.
@@ -321,19 +411,19 @@ output_options.add_argument(
     It is useful also without --pretty: It ensures that the output is flushed
     more often and in smaller chunks.
 
-    """
+    '''
 )
 output_options.add_argument(
     '--output', '-o',
     type=FileType('a+b'),
     dest='output_file',
     metavar='FILE',
-    help="""
+    help='''
     Save output to FILE instead of stdout. If --download is also set, then only
     the response body is saved to FILE. Other parts of the HTTP exchange are
     printed to stderr.
 
-    """
+    '''
 
 )
 
@@ -341,12 +431,12 @@ output_options.add_argument(
     '--download', '-d',
     action='store_true',
     default=False,
-    help="""
+    help='''
     Do not print the response body to stdout. Rather, download it and store it
     in a file. The filename is guessed unless specified with --output
     [filename]. This action is similar to the default behaviour of wget.
 
-    """
+    '''
 )
 
 output_options.add_argument(
@@ -354,11 +444,22 @@ output_options.add_argument(
     dest='download_resume',
     action='store_true',
     default=False,
-    help="""
+    help='''
     Resume an interrupted download. Note that the --output option needs to be
     specified as well.
 
-    """
+    '''
+)
+
+output_options.add_argument(
+    '--quiet', '-q',
+    action='store_true',
+    default=False,
+    help='''
+    Do not print to stdout or stderr.
+    stdout is still redirected if --output is specified.
+    Flag doesn't affect behaviour of download beyond not printing to terminal.
+    '''
 )
 
 #######################################################################
@@ -376,7 +477,7 @@ sessions.add_argument(
     '--session',
     metavar='SESSION_NAME_OR_PATH',
     type=session_name_validator,
-    help=f"""
+    help=f'''
     Create, or reuse and update a session. Within a session, custom headers,
     auth credential, as well as any cookies sent by the server persist between
     requests.
@@ -385,17 +486,17 @@ sessions.add_argument(
 
         {DEFAULT_SESSIONS_DIR}/<HOST>/<SESSION_NAME>.json.
 
-    """
+    '''
 )
 sessions.add_argument(
     '--session-read-only',
     metavar='SESSION_NAME_OR_PATH',
     type=session_name_validator,
-    help="""
+    help='''
     Create or read a session without updating it form the request/response
     exchange.
 
-    """
+    '''
 )
 
 #######################################################################
@@ -408,11 +509,11 @@ auth.add_argument(
     '--auth', '-a',
     default=None,
     metavar='USER[:PASS]',
-    help="""
+    help='''
     If only the username is provided (-a username), HTTPie will prompt
     for the password.
 
-    """,
+    ''',
 )
 
 
@@ -431,12 +532,12 @@ auth.add_argument(
     '--auth-type', '-A',
     choices=_AuthTypeLazyChoices(),
     default=None,
-    help="""
+    help='''
     The authentication mechanism to be used. Defaults to "{default}".
 
     {types}
 
-    """.format(default=_auth_plugins[0].auth_type, types='\n    '.join(
+    '''.format(default=_auth_plugins[0].auth_type, types='\n    '.join(
         '"{type}": {name}{package}{description}'.format(
             type=plugin.auth_type,
             name=plugin.name,
@@ -456,10 +557,10 @@ auth.add_argument(
     '--ignore-netrc',
     default=False,
     action='store_true',
-    help="""
+    help='''
     Ignore credentials from .netrc.
 
-    """,
+    ''',
 )
 
 #######################################################################
@@ -472,9 +573,9 @@ network.add_argument(
     '--offline',
     default=False,
     action='store_true',
-    help="""
+    help='''
     Build the request and print it but don’t actually send it.
-    """
+    '''
 )
 network.add_argument(
     '--proxy',
@@ -482,43 +583,43 @@ network.add_argument(
     action='append',
     metavar='PROTOCOL:PROXY_URL',
     type=KeyValueArgType(SEPARATOR_PROXY),
-    help="""
+    help='''
     String mapping protocol to the URL of the proxy
     (e.g. http:http://foo.bar:3128). You can specify multiple proxies with
     different protocols. The environment variables $ALL_PROXY, $HTTP_PROXY,
     and $HTTPS_proxy are supported as well.
 
-    """
+    '''
 )
 network.add_argument(
     '--follow', '-F',
     default=False,
     action='store_true',
-    help="""
+    help='''
     Follow 30x Location redirects.
 
-    """
+    '''
 )
 
 network.add_argument(
     '--max-redirects',
     type=int,
     default=30,
-    help="""
+    help='''
     By default, requests have a limit of 30 redirects (works with --follow).
 
-    """
+    '''
 )
 
 network.add_argument(
     '--max-headers',
     type=int,
     default=0,
-    help="""
+    help='''
     The maximum number of response headers to be read before giving up
     (default 0, i.e., no limit).
 
-    """
+    '''
 )
 
 network.add_argument(
@@ -526,7 +627,7 @@ network.add_argument(
     type=float,
     default=0,
     metavar='SECONDS',
-    help="""
+    help='''
     The connection timeout of the request in seconds.
     The default value is 0, i.e., there is no timeout limit.
     This is not a time limit on the entire response download;
@@ -534,13 +635,13 @@ network.add_argument(
     timeout seconds (more precisely, if no bytes have been received on
     the underlying socket for timeout seconds).
 
-    """
+    '''
 )
 network.add_argument(
     '--check-status',
     default=False,
     action='store_true',
-    help="""
+    help='''
     By default, HTTPie exits with 0 when no network or other fatal errors
     occur. This flag instructs HTTPie to also check the HTTP status code and
     exit with an error if the status indicates one.
@@ -550,15 +651,24 @@ network.add_argument(
     3xx (Redirect) and --follow hasn't been set, then the exit status is 3.
     Also an error message is written to stderr if stdout is redirected.
 
-    """
+    '''
 )
 network.add_argument(
     '--path-as-is',
     default=False,
     action='store_true',
-    help="""
+    help='''
     Bypass dot segment (/../ or /./) URL squashing.
 
+    '''
+)
+
+network.add_argument(
+    '--chunked',
+    default=False,
+    action='store_true',
+    help="""
+
     """
 )
 
@@ -570,47 +680,58 @@ ssl = parser.add_argument_group(title='SSL')
 ssl.add_argument(
     '--verify',
     default='yes',
-    help="""
+    help='''
     Set to "no" (or "false") to skip checking the host's SSL certificate.
     Defaults to "yes" ("true"). You can also pass the path to a CA_BUNDLE file
     for private certs. (Or you can set the REQUESTS_CA_BUNDLE environment
     variable instead.)
-    """
+    '''
 )
 ssl.add_argument(
-    '--ssl',  # TODO: Maybe something more general, such as --secure-protocol?
+    '--ssl',
     dest='ssl_version',
-    choices=list(sorted(SSL_VERSION_ARG_MAPPING.keys())),
-    help="""
+    choices=list(sorted(AVAILABLE_SSL_VERSION_ARG_MAPPING.keys())),
+    help='''
     The desired protocol version to use. This will default to
     SSL v2.3 which will negotiate the highest protocol that both
     the server and your installation of OpenSSL support. Available protocols
     may vary depending on OpenSSL installation (only the supported ones
     are shown here).
 
-    """
+    '''
+)
+ssl.add_argument(
+    '--ciphers',
+    help=f'''
+
+    A string in the OpenSSL cipher list format. By default, the following
+    is used:
+
+    {DEFAULT_SSL_CIPHERS}
+
+    '''
 )
 ssl.add_argument(
     '--cert',
     default=None,
     type=readable_file_arg,
-    help="""
+    help='''
     You can specify a local cert to use as client side SSL certificate.
     This file may either contain both private key and certificate or you may
     specify --cert-key separately.
 
-    """
+    '''
 )
 
 ssl.add_argument(
     '--cert-key',
     default=None,
     type=readable_file_arg,
-    help="""
+    help='''
     The private key to use with SSL. Only needed if --cert is given and the
     certificate file does not contain the private key.
 
-    """
+    '''
 )
 
 #######################################################################
@@ -623,53 +744,53 @@ troubleshooting.add_argument(
     '--ignore-stdin', '-I',
     action='store_true',
     default=False,
-    help="""
+    help='''
     Do not attempt to read stdin.
 
-    """
+    '''
 )
 troubleshooting.add_argument(
     '--help',
     action='help',
     default=SUPPRESS,
-    help="""
+    help='''
     Show this help message and exit.
 
-    """
+    '''
 )
 troubleshooting.add_argument(
     '--version',
     action='version',
     version=__version__,
-    help="""
+    help='''
     Show version and exit.
 
-    """
+    '''
 )
 troubleshooting.add_argument(
     '--traceback',
     action='store_true',
     default=False,
-    help="""
+    help='''
     Prints the exception traceback should one occur.
 
-    """
+    '''
 )
 troubleshooting.add_argument(
     '--default-scheme',
     default="http",
-    help="""
+    help='''
     The default scheme to use if not specified in the URL.
 
-    """
+    '''
 )
 troubleshooting.add_argument(
     '--debug',
     action='store_true',
     default=False,
-    help="""
+    help='''
     Prints the exception traceback should one occur, as well as other
     information useful for debugging HTTPie itself and for reporting bugs.
 
-    """
+    '''
 )
